      program gscconv

c This program converts GSC star coordinates and magnitudes downloaded
c from the GSC 1.2 web page at STScI, 
c
c    http://www-gsss.stsci.edu/gsc/gsc12/gsc12_form.html
c
c to the format used by the program Astrometrica, by H. Raab.
c
c See the file readme.txt for instructions about how to use the program.
c Please direct questions and bug reports to Paulo R. Holvorcem
c (holvorcem@mpc.com.br)
c
c This program may be distributed freely.
c
c Last revision: 1999 04 27
       
      character*1  sig, file1
      character*2  buff2
      character*12 filein, fileout

      integer km, ns, ra, de, i, j, k, senal, im,
     &        idreg, idord,
     &        nraw, nrawmax, raraw, deraw, nsraw,
     &        idxpos, idxmag, nentr, ientr,
     &        hour, minra, deg, minde,
     &        hour10, hour1, mnra10, mnra1, scra1, scra10,
     &        deg10, deg1, mnde10, mnde1, scde1, scde10

      double precision magraw, secra, secde, mag,
     &                 erpos, ermag, erposraw, ermagraw

      parameter(
     &  km=1000, 
     &  nrawmax=2*km)

      dimension
     &  file1(12),
     &  raraw(nrawmax), 
     &  deraw(nrawmax),
     &  magraw(nrawmax),
     &  nsraw(nrawmax), 
     &  erposraw(nrawmax), 
     &  ermagraw(nrawmax),
     &  idxpos(km), 
     &  idxmag(km)

c Input of file names
      open(1, file='gscconv.ins', status='old')

      do 10 i=1,7
        read(1,'(A)') buff2
   10 continue

   11 read(1,'(A2, 8A1)') buff2, (file1(j), j=1,8)

      if (buff2 .eq. 'EN') goto 12

      k=0
      do 13 j=1,8
        if ((file1(j) .eq. ' ') .or. (file1(j) .eq. '.')) goto 14
        k=k+1
   13 continue

   14 open(2, file='gsctmp.tmp', status='replace')

      file1(k+1)='.'
      file1(k+2)='r'
      file1(k+3)='a'
      file1(k+4)='w'

      write(2,'(12A1)') (file1(j), j=1,k+4)

      file1(k+1)='.'
      file1(k+2)='g'
      file1(k+3)='1'
      file1(k+4)='2'

      write(2,'(12A1)') (file1(j), j=1,k+4)

      close(2, status='keep')

      open(2, file='gsctmp.tmp', status='old')

      read(2,'(A)') filein
      read(2,'(A)') fileout

      close(2, status='delete')

c Read data from GSC 'raw' file

      open(3, file=filein, status='old')

      read(3,'(A)') buff2
      read(3,'(A)') buff2
      read(3,'(A)') buff2
      read(3,100) nentr
      read(3,'(A)') buff2
      read(3,'(A)') buff2

      nraw=0

      do 4 ientr=1,nentr

        read(3,110) idreg, idord, hour, minra, secra, sig, deg, 
     &              minde, secde, erpos, mag, ermag

        ns=idreg*10000 + idord

        nraw=nraw+1
        if (nraw .gt. nrawmax) then
          write(*,*) 'Error: too many entries in raw GSC file.'
          stop
        endif

        nsraw(nraw)=ns

        raraw(nraw)=((hour*60+minra)*60+secra)*1500

        if (sig .eq. '+') then
          senal=1
        elseif (sig .eq. '-') then
          senal=-1
        else
          write(*,*) 'Error: Invalid char. in decl. sign field.'
          stop
        endif
       
        deraw(nraw)=senal*((deg*60+minde)*60+secde)*100
        erposraw(nraw)=erpos

        magraw(nraw)=mag*10
        ermagraw(nraw)=ermag

    4 continue

      close(3, status='keep')

c Choose least-error entries for output

      idxpos(1)=1
      idxmag(1)=1
      im=1

      do 5 ientr=2,nraw
        if (nsraw(ientr) .eq. nsraw(ientr-1)) then
          if (erposraw(ientr) .lt. erposraw(idxpos(im))) 
     &      idxpos(im)=ientr
          if (ermagraw(ientr) .lt. ermagraw(idxmag(im))) 
     &      idxmag(im)=ientr
        else
          im=im+1
          if (im .gt. km) then
            write(*,*) 'Error: too many distinct entries in raw file.'
            stop
          endif
          idxpos(im)=ientr
          idxmag(im)=ientr
        endif
    5 continue 

c Output of uncorrected coordinates and magnitudes

      open(2, file=fileout, status='replace')

      do 6 i=1,im

        idreg=nsraw(idxpos(i))/10000
        idord=nsraw(idxpos(i))-10000*idreg
        ra=raraw(idxpos(i))
        de=deraw(idxpos(i))
        mag=magraw(idxmag(i))/10.

        hour=ra/5400000
        minra=(ra-5400000*hour)/90000
        secra=(ra-5400000*hour-90000*minra)/1500.

        hour10=hour/10
        hour1=hour-10*hour10
        mnra10=minra/10
        mnra1=minra-10*mnra10
        scra10=secra/10
        secra=secra-10*scra10
        scra1=secra
        secra=secra-scra1
        if (secra .ge. 0.995) then
          secra=0.00
          scra1=scra1+1
          if (scra1 .eq. 10) then
            scra1=0
            scra10=scra10+1
            if (scra10 .eq. 6) then
              scra10=0
              mnra1=mnra1+1
              if (mnra1 .eq. 10) then
                mnra1=0
                mnra10=mnra10+1
                if (mnra10 .eq. 6) then
                  mnra10=0
                  hour1=hour1+1
                  if ((hour1 .eq. 4) .and. (hour10 .eq. 2)) then
                    hour1=0
                    hour10=0
                  endif
                  if (hour1 .eq. 10) then
                    hour1=0
                    hour10=hour10+1
                  endif
                endif
              endif
            endif
          endif
        endif

        if (de .ge. 0) then
          sig='+'
        else
          sig='-'
          de=-de
        endif

        deg=de/360000
        minde=(de-360000*deg)/6000
        secde=(de-360000*deg-6000*minde)/100.

        deg10=deg/10
        deg1=deg-10*deg10
        mnde10=minde/10
        mnde1=minde-10*mnde10
        scde10=secde/10
        secde=secde-10*scde10
        scde1=secde
        secde=secde-scde1
        if (secde .ge. 0.95) then
          secde=0.0
          scde1=scde1+1
          if (scde1 .eq. 10) then
            scde1=0
            scde10=scde10+1
            if (scde10 .eq. 6) then
              scde10=0
              mnde1=mnde1+1
              if (mnde1 .eq. 10) then
                mnde1=0
                mnde10=mnde10+1
                if (mnde10 .eq. 6) then
                  mnde10=0
                  deg1=deg1+1
                  if (deg1 .eq. 10) then
                    deg1=0
                    deg10=deg10+1
                  endif
                endif
              endif
            endif
          endif
        endif

        write(2,120) idreg, idord, hour10, hour1, mnra10, mnra1,
     &               scra10, scra1, secra, sig, deg10, deg1, mnde10, 
     &               mnde1, scde10, scde1, secde, mag

    6 continue

      close(2,status='keep')

      goto 11

   12 close(1, status='keep')

c Format statements

  100 format(64X,I4)
  110 format(2I8, 1X, I2, 1X, I2, 1X, F5.2, 1X, A1, I2, 1X, I2,  
     &       1X, F4.1, 1X, F4.1, 2X, F5.2, 1X, F4.2)
  120 format('GSC', 1X, I4, ':', I4, 1X, 2I1, 1X, 2I1, 1X, 2I1, F3.2,  
     &       1X, A1, 2I1, 1X, 2I1, 1X, 2I1, F2.1, 1X, F5.2)

      end

